package it.condarelli.e4.resourcemanager.impl;

import it.condarelli.e4.resourcemanager.IResourceManager;
import org.osgi.service.component.ComponentContext;

import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;

public class ResourceManager implements IResourceManager {

	public static final String	PLUGIN_ID	= "it.condarelli.e4.resourcemanager.impl";

	@Override
	public String getFontName(FontData[] fds) {
		return fds[0].toString();
	}

	@Override
	public Font getFont(String name) {
		FontRegistry fr = JFaceResources.getFontRegistry();
		if (!fr.hasValueFor(name)) {
			try {
				FontData fd = new FontData(name);
				fr.put(name, new FontData[] { fd });
			} catch (Exception e) {
				FontData[] fds = fr.getFontData(name);
				name = getFontName(fds);
				fr.put(name, fds);
			}
		}
		Font f = fr.get(name);
		return f;
	}

	@Override
	public String getColorName(RGB rgb) {
		return rgb.toString();
	}

	@Override
	public Color getColor(String name) {
		int R = 0, G = 0, B = 0;
		RGB rgb;
		Pattern p = Pattern.compile("RGB\\s*\\{\\s*(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*(\\d+)\\s*\\}");
		Matcher m = p.matcher(name);
		boolean b = m.matches();
		if (b) {
			R = Integer.valueOf(m.group(1));
			G = Integer.valueOf(m.group(2));
			B = Integer.valueOf(m.group(3));
		}
		rgb = new RGB(R, G, B);
		return getColor(rgb);
	}

	@Override
	public Color getColor(RGB rgb) {
		ColorRegistry cr = JFaceResources.getColorRegistry();
		String name = getColorName(rgb);
		Color c = cr.get(name);
		if (c == null) {
			cr.put(name, rgb);
			c = cr.get(name);
		}
		return c;
	}

	private class AveragingRgbRaster {
		private int	width;
		private int[]	r, g, b, a, cnt;

		// public AveragingRgbRaster(int width) {
		// this(width, false);
		// }

		public AveragingRgbRaster(int width, boolean hasAlpha) {
			this.width = width;
			r = new int[width];
			g = new int[width];
			b = new int[width];
			cnt = new int[width];
			a = hasAlpha ? (new int[width]) : null;
		}

		public void add(int x, RGB rgb, int alpha) {
			r[x] += rgb.red;
			g[x] += rgb.green;
			b[x] += rgb.blue;
			if (a != null)
				a[x] += alpha;
			cnt[x]++;
		}

		// public void add(int x, int rgb) {
		// r[x] += (rgb >> 16) & 0xFF;
		// g[x] += (rgb >> 8) & 0xFF;
		// b[x] += (rgb >> 0) & 0xFF;
		// cnt[x]++;
		// }

		public void output(ImageData outputImage, int y) {
			int rr, gg, bb, /* rgb, */cnt2;

			for (int x = 0; x < width; x++) {
				cnt2 = cnt[x];
				if (cnt2 == 0) // just in case
				{
					outputImage.setPixel(x, y, 0);
					outputImage.setAlpha(x, y, (a == null) ? 0xff : 0);
				} else {
					// output average RGB pixel value
					rr = r[x] / cnt2;
					gg = g[x] / cnt2;
					bb = b[x] / cnt2;
					// rgb = (rr << 16) | (gg << 8) | (bb);
					outputImage.setPixel(x, y, outputImage.palette.getPixel(new RGB(rr, gg, bb)));
					outputImage.setAlpha(x, y, (a == null) ? 0xff : a[x] / cnt2);
				}
				r[x] = g[x] = b[x] = cnt[x] = 0;
				if (a != null)
					a[x] = 0;
			}
		}

	}

	private ImageData createAveragedImage(ImageData inputImage, int dstwidth, int dstheight) {
		int xsrc, ysrc;
		int xdst, ydst;
		int srcwidth = inputImage.width;
		int srcheight = inputImage.height;
		double xerr, yerr;
		double xincr = (double) dstwidth / (double) srcwidth;
		double yincr = (double) dstheight / (double) srcheight;
		AveragingRgbRaster dstRaster = new AveragingRgbRaster(dstwidth, (inputImage.alphaData != null));
		PaletteData palette = new PaletteData(0xFF, 0xFF00, 0xFF0000);
		ImageData newImage = new ImageData(dstwidth, dstheight, 24, palette);
		// ImageData newImage = new ImageData(dstwidth, dstheight,
		// inputImage.depth, /*inputImage.palette*/ new PaletteData);

		ydst = 0;
		for (ysrc = 0, yerr = 0; ysrc < srcheight; ysrc++, yerr += yincr) {
			if (yerr > 1.0) {
				dstRaster.output(newImage, ydst);
				ydst++;
				yerr -= 1.0;
			}

			xdst = 0;
			for (xsrc = 0, xerr = 0; xsrc < srcwidth; xsrc++, xerr += xincr) {
				if (xerr > 1.0) {
					xerr -= 1.0;
					xdst++;
				}

				dstRaster.add(xdst, inputImage.palette.getRGB(inputImage.getPixel(xsrc, ysrc)), inputImage.getAlpha(xsrc, ysrc));
			}
		}

		if (ydst < dstheight)
			dstRaster.output(newImage, ydst);

		return newImage;
	}

	protected ImageDescriptor imageDescriptorFromPlugin(String pluginId, String fn) {
		URL url = FileLocator.find(Platform.getBundle(pluginId), new Path(fn), null);
		ImageDescriptor id = ImageDescriptor.createFromURL(url);
		return id;
	}

	@Override
	public ImageDescriptor getImageDescriptor(String name) {
		String fn = "/icons/" + name + ".png";
		ImageDescriptor mid = imageDescriptorFromPlugin(PLUGIN_ID, fn);
		return mid;
	}

	@Override
	public ImageDescriptor getImageDescriptor(String name, int size) {
		Image img = getImage(name, size);
		return ImageDescriptor.createFromImage(img);
	}

	@Override
	public Image getImage(String name, int size) {
		Image img;
		String n = name + "[" + String.valueOf(size) + "]";
		ImageRegistry reg = JFaceResources.getImageRegistry();
		ImageDescriptor id = reg.getDescriptor(n);
		if (id == null) {
			ImageDescriptor mid = getImageDescriptor(name);
			if (mid == null)
				return null;
			ImageData imgd = mid.getImageData();
			imgd = scale(imgd, size);
			id = ImageDescriptor.createFromImageData(imgd);
			reg.put(n, id);
		}
		img = id.createImage();
		return img;
	}

	@Override
	public ImageData scale(ImageData in, int size) {
		return createAveragedImage(in, size, size);
	}

	@Override
	public Image scale(Image in, int size) {
		ImageData imgd = scale(in.getImageData(), size);
		ImageDescriptor id = ImageDescriptor.createFromImageData(imgd);
		Image out = id.createImage();
		return out;
	}

	public void activate(ComponentContext context) {
		System.out.println("Service ACTIVATED");       
	}
	
}
