package it.condarelli.e4.writer.modelif;

import it.condarelli.writer.model.world.Actor;
import it.condarelli.writer.model.world.Book;
import it.condarelli.writer.model.world.Chapter;
import it.condarelli.writer.model.world.IBase;
import it.condarelli.writer.model.world.Item;
import it.condarelli.writer.model.world.Location;
import it.condarelli.writer.model.world.Scene;
import it.condarelli.writer.model.world.World;
import it.condarelli.writer.model.world.WorldPackage;
import it.condarelli.writer.model.world.WorldPackage.Literals;

import org.eclipse.core.databinding.observable.Diffs;
import org.eclipse.core.databinding.observable.IObservable;
import org.eclipse.core.databinding.observable.list.IListChangeListener;
import org.eclipse.core.databinding.observable.list.IObservableList;
import org.eclipse.core.databinding.observable.list.ListChangeEvent;
import org.eclipse.core.databinding.observable.list.ListDiff;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.core.databinding.observable.masterdetail.IObservableFactory;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.emf.databinding.EMFProperties;
import org.eclipse.emf.databinding.IEMFListProperty;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.jface.databinding.viewers.ObservableListTreeContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.databinding.viewers.TreeStructureAdvisor;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.graphics.Image;

public class WorldObservableProvider implements IObservableFactory {
    private ObservableMapLabelProvider        omlp  = null;
    private ObservableListTreeContentProvider oltcp = null;
    private TreeStructureAdvisor              tsa   = null;

    private IObservableMap[]                  maps;

    public WorldObservableProvider() {
        tsa = new TreeStructureAdvisor() {
            @Override
            public Object getParent(Object element) {
                if (element instanceof World) {
                    return null;
                } else if (element instanceof VirtualEntry) {
                    return null;
                } else if (element instanceof Actor) {
                    return null;
                } else if (element instanceof Book) {
                    return null;
                } else if (element instanceof Chapter) {
                    return null;
                } else if (element instanceof Location) {
                    return null;
                } else if (element instanceof Item) {
                    return null;
                } else if (element instanceof Scene) {
                    return null;
                }

                return null;
            }

            @Override
            public Boolean hasChildren(Object element) {
                if (element instanceof World) {
                    return true;
                } else if (element instanceof VirtualEntry) {
                    return true;
                } else if (element instanceof Actor) {
                    return false;
                } else if (element instanceof Book) {
                    return true;
                } else if (element instanceof Chapter) {
                    return true;
                } else if (element instanceof Location) {
                    return false;
                } else if (element instanceof Item) {
                    return false;
                } else if (element instanceof Scene) {
                    return false;
                }
                return super.hasChildren(element);
            }
        };
        oltcp = new ObservableListTreeContentProvider(this, tsa);

        IObservableSet set = oltcp.getKnownElements();
        maps = new IObservableMap[2];
        maps[0] = EMFProperties.value(Literals.IBASE__TITLE).observeDetail(set);
        maps[1] = EMFProperties.value(Literals.ACTOR__NAME).observeDetail(set);
        omlp = new ObservableMapLabelProvider(maps) {

            @Override
            public Image getImage(Object element) {
                if (element instanceof VirtualEntry) {
                    VirtualEntry ve = (VirtualEntry) element;
                    return IconProvider.getIconSmall(ve.label) ;
                }
                return IconProvider.getIconSmall(element);
            }

            @Override
            public String getColumnText(Object element, int columnIndex) {
                if (columnIndex == 0) {
                    if (element instanceof IBase) {
                        Object o = maps[0].get(element);
                        return (o != null) ? o.toString() : "(null)";
                    }
                    if (element instanceof VirtualEntry) {
                        return ((VirtualEntry) element).label;
                    }
                    if (element instanceof Actor) {
                        Object o = maps[1].get(element);
                        return (o != null) ? o.toString() : "(null)";
                    }
                }
                return super.getColumnText(element, columnIndex);
            }
        };
        IconProvider.add("Actors", "Actors");
        IconProvider.add("Places", "CompassBook");
        IconProvider.add("Items", "Things");
    }

    public class VirtualEntry extends EObjectImpl {
        private IObservableList list;
        private String          label;

        public VirtualEntry(String label, IObservableList list) {
            this.list = list;
            this.label = label;
        }

        public IObservableList getList() {
            return this.list;
        }

        public String getLabel() {
            return this.label;
        }
    }

    private IEMFListProperty     wActor    = EMFProperties.list(WorldPackage.Literals.WORLD__ACTOR);
    private IEMFListProperty     wLocation = EMFProperties.list(WorldPackage.Literals.WORLD__LOCATION);
    private IEMFListProperty     wBook     = EMFProperties.list(WorldPackage.Literals.WORLD__BOOK);

    private IEMFListProperty     bActor    = EMFProperties.list(WorldPackage.Literals.BOOK__ACTOR);
    private IEMFListProperty     bLocation = EMFProperties.list(WorldPackage.Literals.BOOK__LOCATION);
    private IEMFListProperty     bItem     = EMFProperties.list(WorldPackage.Literals.BOOK__ITEM);
    private IEMFListProperty     bChapter  = EMFProperties.list(WorldPackage.Literals.BOOK__CHAPTER);

    private IEMFListProperty     cActor    = EMFProperties.list(WorldPackage.Literals.CHAPTER__ACTOR);
    private IEMFListProperty     cScene    = EMFProperties.list(WorldPackage.Literals.CHAPTER__SCENE);

    private static final boolean useTom    = true;

    @Override
    public IObservable createObservable(Object target) {
        if (target instanceof IObservableList) {
            return (IObservable) target;
        } else if (target instanceof World) {
            final WritableList list = new WritableList();
            if (useTom) {
                list.add(new VirtualEntry("Actors", wActor.observe(target)));
                list.add(new VirtualEntry("Places", wLocation.observe(target)));
                IObservableList l = wBook.observe(target);
                final IListChangeListener listener = new IListChangeListener() {
                    public void handleListChange(ListChangeEvent event) {
                        if (!list.isDisposed()) {
                            ListDiff diff = Diffs.computeListDiff(list, event.getObservableList());
                            diff.applyTo(list);
                        }
                    }
                };
                l.addListChangeListener(listener);
                list.addAll(wBook.observe(target));
            } else {
                list.add(new VirtualEntry("books", wBook.observe(target)));
                list.add(new VirtualEntry("Actors", wActor.observe(target)));
                list.add(new VirtualEntry("Places", wLocation.observe(target)));
            }
            return list;
        } else if (target instanceof Book) {
            final WritableList list = new WritableList();
            if (useTom) {
                list.add(new VirtualEntry("Actors", bActor.observe(target)));
                list.add(new VirtualEntry("Places", bLocation.observe(target)));
                list.add(new VirtualEntry("Items", bItem.observe(target)));
                IObservableList l = bChapter.observe(target);
                final IListChangeListener listener = new IListChangeListener() {
                    public void handleListChange(ListChangeEvent event) {
                        if (!list.isDisposed()) {
                            ListDiff diff = Diffs.computeListDiff(list, event.getObservableList());
                            diff.applyTo(list);
                        }
                    }
                };
                l.addListChangeListener(listener);
                list.addAll(bChapter.observe(target));
            } else {
                list.add(new VirtualEntry("chapters", bChapter.observe(target)));
                list.add(new VirtualEntry("Actors", bActor.observe(target)));
                list.add(new VirtualEntry("Places", bLocation.observe(target)));
                list.add(new VirtualEntry("Items", bItem.observe(target)));
            }
            return list;
        } else if (target instanceof Chapter) {
            final WritableList list = new WritableList();
            if (useTom) {
                list.add(new VirtualEntry("Actors", cActor.observe(target)));
                IObservableList l = cScene.observe(target);
                final IListChangeListener listener = new IListChangeListener() {
                    public void handleListChange(ListChangeEvent event) {
                        if (!list.isDisposed()) {
                            ListDiff diff = Diffs.computeListDiff(list, event.getObservableList());
                            diff.applyTo(list);
                        }
                    }
                };
                l.addListChangeListener(listener);
                list.addAll(cScene.observe(target));
            } else {
                list.add(new VirtualEntry("scenes", cScene.observe(target)));
                list.add(new VirtualEntry("Actors", cActor.observe(target)));
            }
            return list;
        } else if (target instanceof VirtualEntry) {
            return ((VirtualEntry) target).getList();
        }
        return null;
    }

    public ILabelProvider getLabelProvider() {
        return omlp;
    }

    public IContentProvider getContentProvider() {
        return oltcp;
    }

}
