package it.condarelli.e4.writer.ui.composites;

import it.condarelli.writer.model.world.Book;
import it.condarelli.writer.model.world.WorldPackage;
import it.condarelli.writer.model.world.WorldPackage.Literals;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.emf.databinding.edit.EMFEditProperties;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.databinding.swt.IWidgetValueProperty;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapCellLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ExpandBar;
import org.eclipse.swt.widgets.ExpandItem;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class BookComposite extends PartComposite {

	private Text					txtTitle;
	private TableViewer			tabViewer;
	private Text					txtDesc;
	private Text					txtWords;
	private ExpandItem			xpiChapters;
	private Composite				cmpChapters;

	public BookComposite(Composite parent) {
		super(parent, Book.class);
	}

	/**
	 * @wbp.parser.entryPoint
	 */
	@Override
	protected void createContents(final Composite parent) {

		ExpandBar expandBar = new RadioExpandBar(parent, SWT.MAX);
		expandBar.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		final ExpandItem xpiOverview = new ExpandItem(expandBar, SWT.NONE);
		xpiOverview.setExpanded(true);
		xpiOverview.setText("Overview");
		//
		Composite cmpOverview = new Composite(expandBar, SWT.NONE);
		xpiOverview.setControl(cmpOverview);
		cmpOverview.setLayout(new GridLayout(2, false));
		//
		Label lblTitle = new Label(cmpOverview, SWT.NONE);
		lblTitle.setText("Title:");
		lblTitle.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		//
		txtTitle = new Text(cmpOverview, SWT.NONE);
		txtTitle.setText("[Title]");
		txtTitle.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		//
		txtDesc = new Text(cmpOverview, SWT.WRAP | SWT.MULTI);
		txtDesc.setText("[Desc]");
		txtDesc.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		//
		Label lblWords = new Label(cmpOverview, SWT.NONE);
		lblWords.setText("Words:");
		lblWords.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		//
		txtWords = new Text(cmpOverview, SWT.NONE);
		txtWords.setText("[Words]");
		txtWords.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		xpiOverview.setHeight(xpiOverview.getControl().computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		//
		// txtDesc.addListener(SWT.Modify, new SizeListener(xpndtmNewExpanditem));
		//
		xpiChapters = new ExpandItem(expandBar, SWT.NONE);
		xpiChapters.setExpanded(true);
		xpiChapters.setText("Chapters");
		//
		cmpChapters = new Composite(expandBar, SWT.NONE);
		xpiChapters.setControl(cmpChapters);
		xpiChapters.setHeight(128);
		cmpChapters.setLayout(new GridLayout(1, false));
		//
		tabViewer = new TableViewer(cmpChapters, SWT.SINGLE);
		tabViewer.getTable().setHeaderVisible(true);
		tabViewer.getTable().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		//
	}

	@Override
	protected void createDataBindings(EditingDomain editingDomain, DataBindingContext bindingContext) {
		IWidgetValueProperty txt = WidgetProperties.text(SWT.Modify);
		// IWidgetListProperty lst = WidgetProperties.items();
		// IWidgetValueProperty sel = WidgetProperties.selection();
		// IViewerValueProperty vsel = ViewerProperties.singleSelection();
		//
		bindingContext.bindValue(txt.observe(txtTitle), EMFEditProperties.value(editingDomain, Literals.IBASE__TITLE).observeDetail(oValue));
		bindingContext.bindValue(txt.observe(txtDesc), EMFEditProperties.value(editingDomain, Literals.IBASE__DESC).observeDetail(oValue));
		bindingContext.bindValue(txt.observe(txtWords), EMFEditProperties.value(editingDomain, Literals.BOOK__WORD_COUNT).observeDetail(oValue));
		//
		ObservableListContentProvider cp = new ObservableListContentProvider();
		IObservableMap[] attributeMap = new IObservableMap[2];
		attributeMap[0] = EMFEditProperties.value(editingDomain, WorldPackage.Literals.IBASE__TITLE).observeDetail(cp.getKnownElements());
		attributeMap[1] = EMFEditProperties.value(editingDomain, WorldPackage.Literals.IBASE__DESC).observeDetail(cp.getKnownElements());

		TableViewerColumn column = new TableViewerColumn(tabViewer, SWT.NONE);
		column.getColumn().setText("title");
		column.getColumn().setWidth(100);
		column.setLabelProvider(new ObservableMapCellLabelProvider(attributeMap[0]));

		column = new TableViewerColumn(tabViewer, SWT.NONE);
		column.getColumn().setText("End");
		column.getColumn().setWidth(500);
		column.setLabelProvider(new ObservableMapCellLabelProvider(attributeMap[1]));

		tabViewer.setContentProvider(cp);
		tabViewer.setInput(EMFEditProperties.list(editingDomain, WorldPackage.Literals.BOOK__CHAPTER).observeDetail(oValue));
	}

	@Override
	public void setFocus() {
		txtDesc.setFocus();
	}

	public void setBook(Book book) {
		setValue(book);
	}

}
