package it.condarelli.e4.writer.ui.composites;

import it.condarelli.writer.model.world.Chapter;
import it.condarelli.writer.model.world.WorldPackage;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.databinding.observable.list.IObservableList;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.emf.databinding.EMFObservables;
import org.eclipse.emf.databinding.edit.EMFEditProperties;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.databinding.swt.IWidgetValueProperty;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.ExpandBar;
import org.eclipse.swt.widgets.ExpandItem;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Scrollable;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

public class ChapterComposite extends PartComposite {

	private final class SizeListener implements Listener {
		private final ExpandItem	exI;

		private SizeListener(ExpandItem xpndtmNewExpanditem) {
			this.exI = xpndtmNewExpanditem;
		}

		@Override
		public void handleEvent(Event event) {
			Display.getCurrent().asyncExec(new Runnable() {
				public void run() {
					Table t = (Table) tableViewer.getControl();
					if (t.isDisposed())
						return;
					final int TRIAL_WIDTH = 100;
					final int trimWidth = t.computeTrim(0, 0, TRIAL_WIDTH, 100).width - TRIAL_WIDTH;
					Point size = t.computeSize(t.getSize().x - trimWidth, SWT.DEFAULT);
					if (exI.getHeight() != size.y) {
						exI.setHeight(size.y);
					}
				}
			});
		}
	}

	public void resize(final Scrollable master, final ExpandItem slave) {
		Display.getCurrent().asyncExec(new Runnable() {
			public void run() {
				if (master.isDisposed())
					return;
				final int TRIAL_WIDTH = 100;
				final int trimWidth = master.computeTrim(0, 0, TRIAL_WIDTH, 100).width - TRIAL_WIDTH;
				Point size = master.computeSize(master.getSize().x - trimWidth, SWT.DEFAULT);
				if (slave.getHeight() != size.y) {
					slave.setHeight(size.y);
				}
			}
		});
	}

	private Text			txtTitle;
	private Text			txtDesc;
	private Text			txtWords;
	private TableViewer	tableViewer;
	private ExpandItem	xpndtmScenes;
	private Composite		cmpScenes;

	public ChapterComposite(final Composite parent) {
		super(parent, Chapter.class);
	}

	@Override
	public void setFocus() {
		txtTitle.setFocus();
	}

	public void setChapter(Chapter chapter) {
		setValue(chapter);
	}

	/**
	 * @wbp.parser.entryPoint
	 */
	@Override
	protected void createContents(Composite parent) {
		parent.setLayout(new GridLayout(1, false));

		ExpandBar expandBar = new RadioExpandBar(parent, SWT.NONE);
		expandBar.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		ExpandItem xpndtmOverview = new ExpandItem(expandBar, SWT.NONE);
		xpndtmOverview.setExpanded(true);
		xpndtmOverview.setText("Overview");

		Composite cmpOverview = new Composite(expandBar, SWT.NONE);
		xpndtmOverview.setControl(cmpOverview);
		xpndtmOverview.setHeight(128);
		cmpOverview.setLayout(new GridLayout(2, false));

		Label lblTitle = new Label(cmpOverview, SWT.NONE);
		lblTitle.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblTitle.setText("Title:");

		txtTitle = new Text(cmpOverview, SWT.BORDER);
		txtTitle.setText("[Title]");
		txtTitle.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		txtDesc = new Text(cmpOverview, SWT.WRAP | SWT.V_SCROLL | SWT.MULTI);
		txtDesc.setText("[Desc]");
		txtDesc.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));

		Label lblWords = new Label(cmpOverview, SWT.NONE);
		lblWords.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblWords.setText("Words:");

		txtWords = new Text(cmpOverview, SWT.BORDER);
		txtWords.setText("[Words]");
		txtWords.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		xpndtmScenes = new ExpandItem(expandBar, SWT.NONE);
		xpndtmScenes.setExpanded(true);
		xpndtmScenes.setText("Scenes");

		cmpScenes = new Composite(expandBar, SWT.NONE);
		xpndtmScenes.setControl(cmpScenes);
		cmpScenes.setLayout(new GridLayout(1, false));

		tableViewer = new TableViewer(cmpScenes, SWT.BORDER | SWT.FULL_SELECTION);
		Table table = tableViewer.getTable();
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		table.setHeaderVisible(true);

		TableViewerColumn tableViewerColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		TableColumn tblclmnTitle = tableViewerColumn.getColumn();
		tblclmnTitle.setWidth(100);
		tblclmnTitle.setText("Title");

		TableViewerColumn tableViewerColumn_1 = new TableViewerColumn(tableViewer, SWT.NONE);
		TableColumn tblclmnGoal = tableViewerColumn_1.getColumn();
		tblclmnGoal.setWidth(100);
		tblclmnGoal.setText("Goal");

		TableViewerColumn tableViewerColumn_2 = new TableViewerColumn(tableViewer, SWT.NONE);
		TableColumn tblclmnDescrizione = tableViewerColumn_2.getColumn();
		tblclmnDescrizione.setWidth(500);
		tblclmnDescrizione.setText("Descrizione");
		// xpndtmScenes.setHeight(xpndtmScenes.getControl().computeSize(SWT.DEFAULT,
		// SWT.DEFAULT).y);
		// table.addListener(SWT.Modify, new Listener() {
		//
		// @Override
		// public void handleEvent(Event event) {
		// Display.getCurrent().asyncExec(new Runnable() {
		// public void run() {
		// parent.layout();
		// }
		// });
		// }
		// });
		table.addListener(SWT.Modify, new SizeListener(xpndtmScenes));
		xpndtmScenes.setHeight(200);
	}

	@Override
	protected void createDataBindings(EditingDomain editingDomain, DataBindingContext bindingContext) {
		IWidgetValueProperty txt = WidgetProperties.text(SWT.Modify);
		//
		bindingContext.bindValue(txt.observeDelayed(400, txtTitle), EMFEditProperties.value(editingDomain, WorldPackage.Literals.IBASE__TITLE).observeDetail(oValue));
		bindingContext.bindValue(txt.observeDelayed(400, txtDesc), EMFEditProperties.value(editingDomain, WorldPackage.Literals.IBASE__DESC).observeDetail(oValue));
		bindingContext.bindValue(txt.observeDelayed(400, txtWords), EMFEditProperties.value(editingDomain, WorldPackage.Literals.CHAPTER__WORD_COUNT).observeDetail(oValue));

		ObservableListContentProvider lcp = new ObservableListContentProvider();
		IObservableMap[] observeMaps = EMFObservables.observeMaps(lcp.getKnownElements(), new EStructuralFeature[] { WorldPackage.Literals.IBASE__TITLE,
				WorldPackage.Literals.SCENE__GOAL, WorldPackage.Literals.IBASE__DESC });
		tableViewer.setLabelProvider(new ObservableMapLabelProvider(observeMaps));
		tableViewer.setContentProvider(lcp);
		IObservableList iol = EMFObservables.observeDetailList(Realm.getDefault(), oValue, WorldPackage.Literals.CHAPTER__SCENE);
		tableViewer.setInput(iol);
	}
}
