package it.condarelli.e4.writer.ui.composites;

import it.condarelli.writer.model.world.IBase;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.observable.value.WritableValue;
import org.eclipse.emf.common.command.CommandStack;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ExpandBar;

public abstract class PartComposite {

	protected IObservableValue	oValue;
	private EditingDomain		editingDomain;
	private ExpandBar				expandBar;

	public PartComposite(final Composite parent, Class<? extends EObject> clazz) {
		oValue = new WritableValue(null, clazz);
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		parent.setLayout(gl);
		//
		createContents(parent);
		createDataBindings();
	}

	protected abstract void createContents(final Composite parent);

	protected void createDataBindings() {
		if (editingDomain == null)
			return;

		DataBindingContext dbc = new DataBindingContext();
		createDataBindings(editingDomain, dbc);
	}

	protected abstract void createDataBindings(EditingDomain editingDomain, DataBindingContext bindingContext);

	public void setValue(IBase base) {
		if (oValue.getValue() != base) {
			oValue.setValue(base);
			EditingDomain ed = base.getRoot().getModel().getEditingDomain();
			if (ed != editingDomain) {
				editingDomain = ed;
				createDataBindings();
			}
			update();
		}
	}

	public IBase getValue() {
		return (IBase) oValue.getValue();
	}
	
	public boolean isDirty() {
		CommandStack commandStack = editingDomain.getCommandStack();
		return commandStack.canUndo();
	}
	
	protected void update() {
	}

	public void setFocus() {
		expandBar.setFocus();
	}
}
