package it.condarelli.e4.writer.ui.composites;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ExpandEvent;
import org.eclipse.swt.events.ExpandListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.ExpandBar;
import org.eclipse.swt.widgets.ExpandItem;
import org.eclipse.swt.widgets.Scrollable;

public class RadioExpandBar extends ExpandBar implements ExpandListener, PaintListener {

	private boolean isRadio() {
		return (getStyle() & SWT.RADIO) != 0;
	}

	private boolean isResize() {
		return (getStyle() & SWT.NO_TRIM) == 0;
	}

	public RadioExpandBar(Composite parent, int style) {
		super(parent, style);
		addExpandListener(this);
		addPaintListener(this);
	}

	@Override
	public void itemCollapsed(ExpandEvent e) {
		if (e.widget instanceof ExpandBar) {
			ExpandBar eb = (ExpandBar) e.widget;
			if (e.item instanceof ExpandItem) {
				ExpandItem ei = (ExpandItem) e.item;
				check(eb, ei);
			}
		}
	}

	@Override
	public void itemExpanded(ExpandEvent e) {
		if (e.widget instanceof ExpandBar) {
			ExpandBar eb = (ExpandBar) e.widget;
			if (e.item instanceof ExpandItem) {
				ExpandItem ei = (ExpandItem) e.item;
				check(eb, ei);
			}
		}
	}

	@Override
	protected void checkSubclass() {
	}

	private boolean repainting = false;
	
	@Override
	public void paintControl(PaintEvent e) {
		if (repainting)
			return;
		repainting = true;
		if (e.widget instanceof ExpandBar) {
			ExpandBar eb = (ExpandBar) e.widget;
			check(eb, null);
		}
	}

	private void check(final ExpandBar eb, final ExpandItem ei0) {
		Display.getCurrent().asyncExec(new Runnable() {
			public void run() {
				if (eb.isDisposed())
					return;
				if ((ei0 != null) && ei0.isDisposed())
					return;
				eb.getParent().layout();
				//System.out.println(String.format("RadioExpandBar.check('%s', '%s'): h=%3d (%3d)", 
				//		eb.getToolTipText(), (ei0==null)? "(null)": ei0.getText(), eb.getClientArea().height, eb.getParent().getClientArea().height));
				ExpandItem ei = ei0;
				int spacing = eb.getSpacing();
				int headers = spacing;
				//System.out.println(String.format("-----h=%3d", headers));
				for (ExpandItem i : eb.getItems()) {
					if (i.getExpanded()) {
						if (ei == null) {
							ei = i;
						} else {
							if (isRadio() && (ei != i)) {
								i.setExpanded(false);
							}
						}
					}
					headers += i.getHeaderHeight() + spacing;
					//System.out.println(String.format("     h=%3d", headers));
				}
				if (isResize()) {
					int avail = eb.getClientArea().height - headers;
					for (ExpandItem i : eb.getItems()) {
						if (i.getExpanded()) {
							Control control = i.getControl();
							if (control instanceof Scrollable) {
								Scrollable master = (Scrollable) control;
								if (master.isDisposed())
									return;
								final int TRIAL_WIDTH = 100;
								final int trimWidth = master.computeTrim(0, 0, TRIAL_WIDTH, 100).width - TRIAL_WIDTH;
								int vsize = master.computeSize(master.getSize().x - trimWidth, SWT.DEFAULT).y;
								int height = i.getHeight();
								int newh = vsize;
								if (newh > avail)
									newh = avail;
								//System.out.println(String.format("          a=%3d; h=%3d; b=%3d", avail, height, newh));
								if (height != newh) {
									i.setHeight(newh);
								}
								avail -= newh;
							}
						}
					}
				}
				eb.update();
				repainting = false;
			}
		});
	}
}
