package it.condarelli.e4.writer.ui.parts;

import java.io.IOException;

import it.condarelli.e4.writer.modelif.WorldObservableProvider;
import it.condarelli.writer.model.world.Book;
import it.condarelli.writer.model.world.Chapter;
import it.condarelli.writer.model.world.Model;
import it.condarelli.writer.model.world.Scene;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.eclipse.core.databinding.observable.list.IObservableList;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.emf.databinding.EMFObservables;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;

public class MasterPart {
	public static final String	ID	= "it.condarelli.e4.writer.ui.parts.master";

	@Inject
	MDirtyable						dirtyable;

	@Inject
	private ESelectionService	selectionService;

	// private Details details = null;

	private Model					model;
	private TreeViewer			treeViewer;
	private Tree					tree;

	@Inject
	public MasterPart() {
		System.out.println("MasterPart.MasterPart():");
	}

	@Inject
	private EPartService	partService;

	@PostConstruct
	public void createControls(Composite parent, @Optional Model model) {
		if (parent == null) {
			System.err.println("No Composite available.");
			return;
		}
		if (treeViewer == null) {
			treeViewer = new TreeViewer(parent, SWT.BORDER);
			treeViewer.setAutoExpandLevel(2);
			tree = treeViewer.getTree();
			tree.setHeaderVisible(true);
			tree.setLinesVisible(true);
			/**/
			treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
				@Override
				public void selectionChanged(SelectionChangedEvent event) {
					IStructuredSelection selection = (IStructuredSelection) event.getSelection();
					final Object o = selection.getFirstElement();
					selectionService.setSelection(o);
					/**/
					String partID = null;
					if (o instanceof Scene) {
						partID = ScenePart.ID;
					} else if (o instanceof Chapter) {
						partID = ChapterPart.ID;
					} else if (o instanceof Book) {
						partID = BookPart.ID;
					}
					if (partID != null) {
						MPart part = partService.findPart(partID);
						if (part != null) {
							part.setVisible(true);
							partService.showPart(part, PartState.ACTIVATE);
							System.out.println("selectionChanged(): showing part " + part.getLabel());
						}
					}
					/**/
					System.out.println("selectionChanged(): selecting " + o.getClass().getName());
				}
			});
			/**/
		}
		System.out.println("Master.createControls: " + ((model == null) ? "No model available" : model.getClass()));
		setModel(model);
	}

	public void setModel(Model model) {
		if (this.model != model) {
			this.model = model;

			WorldObservableProvider wop = new WorldObservableProvider();
			treeViewer.setContentProvider(wop.getContentProvider());
			treeViewer.setLabelProvider(wop.getLabelProvider());

			Resource r = model.getResource();
			IObservableList iol = EMFObservables.observeResourceContents(r);
			treeViewer.setInput(iol);
			
			model.addDirtyListener(new Model.DirtyListener() {
				
				@Override
				public void setDirty(boolean dirty) {
					dirtyable.setDirty(dirty);
				}
			});
		}
	}

	@Focus
	public void setFocus() {
		tree.setFocus();
	}

	@PreDestroy
	public void dispose() {

	}

	@Persist
	public void save() {
		if (dirtyable.isDirty()) {
			try {
				model.save();
				dirtyable.setDirty(false);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
}
